#import "UdidPlugin.h"
#import "Md5.h"
#import <UDIDEngine.h>
#import <UDIDFaceCompareFactory.h>

#define UDPUBKEY        @"商户 pubkey"       // 格式为：@"12121212-1212-1212-1212-121212121212"
#define UDSECURITYKEY   @"商户 securitykey"

@interface UdidPlugin ()<UDIDEngineDelegate>
@property (nonatomic, copy) NSString * partnerOrderId;
@property (nonatomic, copy) NSString * signTime;
@property (nonatomic, copy) FlutterResult result;
@end

@implementation UdidPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"udid_plugin"
                                     binaryMessenger:[registrar messenger]];
    UIViewController *viewController = [UIApplication sharedApplication].delegate.window.rootViewController;
    UdidPlugin* instance = [[UdidPlugin alloc] initWithViewController:viewController];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (instancetype)initWithViewController:(UIViewController *)viewController {
    self = [super init];
    if (self) {
        self.viewController = viewController;
    }
    return self;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    _result = result;
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } else if ([@"startOCRFlow" isEqualToString:call.method]) {
//      NSDictionary *engineDic = call.arguments;
      [self startOCRFlow];
  } else if ([@"startLivenessFlow" isEqualToString:call.method]) {
      [self startLivenessFlow];
  } else if ([@"startIDAuthFlow" isEqualToString:call.method]) {
      [self startIDAuthFlow];
  } else if ([@"startCompareFlow" isEqualToString:call.method]) {
      [self startCompareFlow];
  } else if ([@"startVideoFlow" isEqualToString:call.method]) {
      [self startVideoFlow];
  } else if ([@"startCustomFlow" isEqualToString:call.method]) {
      [self startCustomFlow];
  } else {
    result(FlutterMethodNotImplemented);
  }
}

#pragma mark - 产品动作

// OCR 身份证扫描
- (void)startOCRFlow {
    UDIDEngine *ocrEngine = [[UDIDEngine alloc] init];
    /* 身份证 OCR 扫描相关参数 */
    ocrEngine.actions = @[@0];
    // 或者下面的 actions 传入方式
    // ocrEngine.actions = @[[NSNumber numberWithUnsignedInteger:UDIDAuthFlowOCR]];
    ocrEngine.showInfo = YES;
    //    ocrEngine.mosaicIdName = NO;
    ocrEngine.mosaicIdNumber = NO;
    ocrEngine.showConfirmIdNumber = NO;
    
    /* 通用参数 */
    ocrEngine.pubKey = UDPUBKEY;
    ocrEngine.signTime = [self getTimeSp];
    self.signTime = ocrEngine.signTime;
    ocrEngine.partnerOrderId = [NSString stringWithFormat:@"%@", [self getTimeSp]];
    self.partnerOrderId = ocrEngine.partnerOrderId;
    //    ocrEngine.notifyUrl = @"你的异步通知地址，请填规范的 URL";
    ocrEngine.sign = [self getSignatureByMd5];
    ocrEngine.delegate = self;
    [ocrEngine startIdSafeAuthInViewController:_viewController];
}

// 活体检测
- (void)startLivenessFlow {
    UDIDEngine *livenessEngine = [[UDIDEngine alloc] init];
    livenessEngine.actions = @[@1];
    // 或者下面的 actions 传入方式
    // livenessEngine.actions = @[[NSNumber numberWithUnsignedInteger:UDIDAuthFlowLiving]];
    
    /* 活体检测相关参数 */
    livenessEngine.livingMode = UDIDLivingCommandMode;
    //    livenessEngine.randomCount = 3;
    //    livenessEngine.livingModeSettings = @[@0, @1, @2, @3, @6];
    /*
     UDIDAuthFlowOCR         = 0,            // 身份证 OCR 扫描
     UDIDAuthFlowLiving      = 1,            // 活体检测
     UDIDAuthFlowCompare     = 2,            // 比对
     UDIDAuthFlowRealName    = 3,            // 实名验证
     UDIDAuthFlowVideo       = 5
     */
    //    livenessEngine.livingModeSettings = @[[NSNumber numberWithUnsignedInteger:UDIDLivingBlink],
    //                                           [NSNumber numberWithUnsignedInteger:UDIDLivingFaceToLeft],
    //                                           [NSNumber numberWithUnsignedInteger:UDIDLivingFaceToRight],
    //                                           [NSNumber numberWithUnsignedInteger:UDIDLivingSwingHead]];
    livenessEngine.closeRemindVoice = NO;
    
    /* 通用参数 */
    livenessEngine.pubKey = UDPUBKEY;
    livenessEngine.signTime = [self getTimeSp];
    self.signTime = livenessEngine.signTime;
    livenessEngine.partnerOrderId = [NSString stringWithFormat:@"%@", [self getTimeSp]];
    self.partnerOrderId = livenessEngine.partnerOrderId;
    //    livenessEngine.notifyUrl = @"你的异步通知地址，请填规范的 URL";
    livenessEngine.sign = [self getSignatureByMd5];
    livenessEngine.delegate = self;
    [livenessEngine startIdSafeAuthInViewController:_viewController];
}

// 实名验证
- (void)startIDAuthFlow {
    UDIDEngine *idauthEngine = [[UDIDEngine alloc] init];
    idauthEngine.actions = @[@3];
    // 或者下面的 actions 传入方式
    // idauthEngine.actions = @[[NSNumber numberWithUnsignedInteger:UDIDAuthFlowRealName]];
    idauthEngine.verifyType = UDIDVerifyHumanType;
    idauthEngine.idName = @"小明";
    idauthEngine.idNumber = @"身份证号码";
    
    /* 通用参数 */
    idauthEngine.pubKey = UDPUBKEY;
    idauthEngine.signTime = [self getTimeSp];
    self.signTime = idauthEngine.signTime;
    idauthEngine.partnerOrderId = [NSString stringWithFormat:@"%@", [self getTimeSp]];
    self.partnerOrderId = idauthEngine.partnerOrderId;
    //    idauthEngine.notifyUrl = @"你的异步通知地址，请填规范的 URL";
    idauthEngine.sign = [self getSignatureByMd5];
    idauthEngine.delegate = self;
    [idauthEngine startIdSafeAuthInViewController:_viewController];
}

// 比对
- (void)startCompareFlow {
    UDIDEngine *compareEngine = [[UDIDEngine alloc] init];
    
    compareEngine.actions = @[@2];
    // 或者下面的 actions 传入方式
    // compareEngine.actions = @[[NSNumber numberWithUnsignedInteger:UDIDAuthFlowCompare]];
    
    compareEngine.compareItemA = [UDIDFaceCompareFactory getBytype:UDIDSafePhotoTypeNormal];;
    compareEngine.compareItemB = [UDIDFaceCompareFactory getBytype:UDIDSafePhotoTypeLiving];;
    compareEngine.isGridPhoto = NO;
    
    /* 通用参数 */
    compareEngine.pubKey = UDPUBKEY;
    compareEngine.signTime = [self getTimeSp];
    self.signTime = compareEngine.signTime;
    compareEngine.partnerOrderId = [NSString stringWithFormat:@"%@", [self getTimeSp]];
    self.partnerOrderId = compareEngine.partnerOrderId;
    //    compareEngine.notifyUrl = @"你的异步通知地址，请填规范的 URL";
    compareEngine.sign = [self getSignatureByMd5];
    compareEngine.delegate = self;
    [compareEngine startIdSafeAuthInViewController:_viewController];
}

// 视频存证
- (void)startVideoFlow {
    UDIDEngine *videoEngine = [[UDIDEngine alloc] init];
    
    videoEngine.actions = @[@5];
    //    videoEngine.actions = @[[NSNumber numberWithUnsignedInteger:UDIDAuthFlowVideo]];
    videoEngine.readingInfo = @"门前大桥下游过一群鸭，快来快来数一数，二四六七八";
    
    /* 通用参数 */
    videoEngine.pubKey = UDPUBKEY;
    videoEngine.signTime = [self getTimeSp];
    self.signTime = videoEngine.signTime;
    videoEngine.partnerOrderId = [NSString stringWithFormat:@"%@", [self getTimeSp]];
    self.partnerOrderId = videoEngine.partnerOrderId;
    //    videoEngine = @"www.baidu.com";
    videoEngine.sign = [self getSignatureByMd5];
    videoEngine.delegate = self;
    [videoEngine startIdSafeAuthInViewController:_viewController];
}


// 自由组合流程 (例子：实名验证+活体检测+人脸比对)
- (void)startCustomFlow {
    UDIDEngine *customEngine = [[UDIDEngine alloc] init];
    customEngine.actions = @[@3, @1, @2];
    // 或者下面的 actions 传入方式
    //    customEngine.actions = @[[NSNumber numberWithUnsignedInteger:UDIDAuthFlowRealName],
    //                              [NSNumber numberWithUnsignedInteger:UDIDAuthFlowLiving],
    //                              [NSNumber numberWithUnsignedInteger:UDIDAuthFlowCompare]];
    
    customEngine.verifyType = UDIDVerifyHumanType;
    customEngine.idName = @"小明";
    customEngine.idNumber = @"身份证号码";
    
    customEngine.randomCount = 3;
    customEngine.livingModeSettings = @[@0, @1, @2, @3, @6];
    customEngine.livingMode = UDIDLivingCustomMode;
    customEngine.closeRemindVoice = YES;
    
    customEngine.compareItemA = [UDIDFaceCompareFactory getBytype:UDIDSafePhotoTypeLiving];
    customEngine.compareItemB = [UDIDFaceCompareFactory getBytype:UDIDSafePhotoTypeNormal];
    customEngine.isGridPhoto = YES;
    
    /* 通用参数 */
    customEngine.pubKey = UDPUBKEY;
    customEngine.signTime = [self getTimeSp];
    self.signTime = customEngine.signTime;
    customEngine.partnerOrderId = [NSString stringWithFormat:@"%@", [self getTimeSp]];
    self.partnerOrderId = customEngine.partnerOrderId;
    //    customEngine.notifyUrl = @"你的异步通知地址，请填规范的 URL";
    customEngine.sign = [self getSignatureByMd5];
    customEngine.delegate = self;
    [customEngine startIdSafeAuthInViewController:_viewController];
}

#pragma mark - Delegate

- (void)idSafeEngineFinishedResult:(UDIDEngineResult)result UserInfo:(id)userInfo {
    
    //    NSLog(@"demo:userInfo = %@", userInfo);
    
    switch (result) {
        case UDIDEngineResult_OCR: {
            // TODO: 身份证OCR识别 回调
            _result(userInfo);
            break;
        } case UDIDEngineResult_Liveness: {
            // TODO: 活体检测 回调
            _result(userInfo);
            break;
        } case UDIDEngineResult_IDAuth: {
            // TODO: 实名验证 回调
            _result(userInfo);
            break;
        } case UDIDEngineResult_FaceCompare: {
            // TODO: 人脸比对 回调
            _result(userInfo);
            break;
        } case UDIDEngineResult_Video: {
            // TODO: 视频存证 回调
            _result(userInfo);
            break;
        } default:
            _result(userInfo);
            break;
    }
}

// 拍照
- (void)startTakePhoto {
    NSLog(@"传入的 base 64 格式照片分辨率宽度不得大于500，具体可以参照压缩图片方法，scaledToWidth:(<500)，在转base64(具体参照 imageToNSString: 方法)");
}

#pragma mark - 签名与签名时间

// 签名方法
- (NSString*)getSignatureByMd5 {
    NSString *resultString = nil;
    NSString* signature = [NSString stringWithFormat:@"pub_key=%@|partner_order_id=%@|sign_time=%@|security_key=%@",UDPUBKEY,self.partnerOrderId,self.signTime,UDSECURITYKEY];
    resultString = [Md5 encodeToLowerCase:signature];
    return resultString;
}

// 签名时间方法
- (NSString *)getTimeSp {
    NSString *resultString = nil;
    NSTimeInterval time = [[NSDate date] timeIntervalSince1970] ;
    NSDateFormatter* formatter = [[NSDateFormatter alloc] init];
    [formatter setDateStyle:NSDateFormatterMediumStyle];
    [formatter setTimeStyle:NSDateFormatterShortStyle];
    [formatter setDateFormat:@"yyyyMMddHHMMss"];
    NSDate *datenow = [NSDate dateWithTimeIntervalSince1970:time];
    resultString = [formatter stringFromDate:datenow];
    return resultString;
}

#pragma mark - 图片转 base64
-(NSString *)imageToNSString:(UIImage *)image {
    NSData *imageData = UIImagePNGRepresentation(image);
    return [imageData base64EncodedStringWithOptions:NSDataBase64Encoding64CharacterLineLength];
}

#pragma mark - base64 转图片
-(UIImage *)stringToUIImage:(NSString *)string {
    NSData *data = [[NSData alloc]initWithBase64EncodedString:string
                                                      options:NSDataBase64DecodingIgnoreUnknownCharacters];
    return [UIImage imageWithData:data];
}

#pragma mark - 压缩图片
- (UIImage*)imageWithImage:(UIImage*)sourceImage scaledToWidth:(float)i_width {
    float oldWidth = sourceImage.size.width;
    float scaleFactor = i_width / oldWidth;
    float newHeight = sourceImage.size.height * scaleFactor;
    float newWidth = oldWidth * scaleFactor;
    UIGraphicsBeginImageContext(CGSizeMake(newWidth, newHeight));
    [sourceImage drawInRect:CGRectMake(0, 0, newWidth, newHeight)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

@end
